class VimModel:
    def __init__(self):
        self.displayBuffer = []  # буфер для хранения всех строк
        self.currentLine = 0  # текущий индекс строки
        self.currentCol = 0  # текущий индекс колонки
        self.scrollY = 0  # вертикальная прокрутка
        self.scrollX = 0  # горизонтальная прокрутка
        self.file_path = ""  # путь к файлу

    def Enter(self) -> None:
        # Разделяем текущую строку на две части
        new_line = self.displayBuffer[self.currentLine][self.currentCol:]
        self.displayBuffer[self.currentLine] = self.displayBuffer[self.currentLine][:self.currentCol]
        self.currentLine += 1  # Переходим на следующую строку
        self.displayBuffer.insert(self.currentLine, new_line)  # Вставляем новую строку
        self.currentCol = 0  # Сбрасываем индекс колонки

    def Backspace(self) -> None:
        if self.currentCol > 0:  # Если символ существует в текущей строке
            self.currentCol -= 1
            del self.displayBuffer[self.currentLine][self.currentCol]  # Удаляем символ
        elif self.currentLine > 0:  # Если текущая строка не первая
            # Объединяем текущую строку с предыдущей
            prev_line_length = len(self.displayBuffer[self.currentLine - 1])
            self.displayBuffer[self.currentLine - 1].extend(self.displayBuffer[self.currentLine])
            del self.displayBuffer[self.currentLine]
            self.currentLine -= 1
            self.currentCol = prev_line_length  # Переходим в конец предыдущей строки

    def MoveLeft(self) -> None:
        if self.currentCol > 0:
            self.currentCol -= 1
        elif self.currentLine > 0:
            self.currentLine -= 1
            self.currentCol = len(self.displayBuffer[self.currentLine])

    def MoveRight(self) -> None:
        if self.currentCol < len(self.displayBuffer[self.currentLine]):
            self.currentCol += 1
        elif self.currentLine < len(self.displayBuffer) - 1:
            self.currentLine += 1
            self.currentCol = 0

    def MoveUp(self) -> None:
        if self.currentLine > 0:
            self.currentLine -= 1
            self.currentCol = min(self.currentCol, len(self.displayBuffer[self.currentLine]))

    def MoveDown(self) -> None:
        if self.currentLine < len(self.displayBuffer) - 1:
            self.currentLine += 1
            self.currentCol = min(self.currentCol, len(self.displayBuffer[self.currentLine]))

    def SaveFile(self) -> None:
        result = self.save_file()
        if result is True:
            print(f"File {self.file_path} saved successfully.")
        else:
            print(f"Error saving file: {result}")

    def load_file(self, file_path) -> None:
        """Загрузка файла для редактирования"""
        self.file_path = file_path
        try:
            with open(file_path, "r") as file:
                self.displayBuffer = [list(line.rstrip('\n')) for line in file.readlines()]
        except FileNotFoundError:
            print(f"File {file_path} not found. Starting with empty buffer.")
            self.displayBuffer = []

    def save_file(self) -> None:
        """Сохранение файла"""
        try:
            with open(self.file_path, "w") as file:
                for line in self.displayBuffer:
                    file.write(''.join(line) + '\n')
            return True
        except Exception as e:
            return str(e)